@@ -1,6 +1,6 @@
 from django.contrib import admin
 
-from whisperer.models import Webhook, WebhookEvent
+from whisperer.models import EventQueue, Webhook, WebhookEvent
 
 
 @admin.register(Webhook)
@@ -11,3 +11,8 @@ class WebhookAdmin(admin.ModelAdmin):
 @admin.register(WebhookEvent)
 class WebhookEventAdmin(admin.ModelAdmin):
     list_display = ("uuid", "webhook", "delivered", "response_http_status")
+
+
+@admin.register(EventQueue)
+class EventQueueAdmin(admin.ModelAdmin):
+    list_display = ("uuid", "hook_id", "event_type")
