@@ -6,6 +6,7 @@ from whisperer.exceptions import AlreadyRegisteredEvent, ImproperlyConfigured
 class WhispererEvent(object):
     serializer_class = None
     event_type = None
+    queryset = None
 
     def _serialize(self, instance):
         objects = [instance] if not isinstance(instance, list) else instance
@@ -21,6 +22,9 @@ class WhispererEvent(object):
         many = isinstance(instance, list)
         return self.serializer_class(instance, many=many).data
 
+    def customize_payload(self, payload):
+        return payload
+
 
 class EventRegistry(object):
     def __init__(self):
@@ -32,6 +36,9 @@ class EventRegistry(object):
     def __getitem__(self, event_type):
         return self._registry[event_type]
 
+    def keys(self):
+        return self._registry.keys()
+
     @property
     def event_types(self):
         return self._registry.keys()
