@@ -2,7 +2,8 @@ from rest_framework import serializers
 from rest_framework.exceptions import ValidationError
 
 from whisperer.countdown import countdown_classes
-from whisperer.models import Webhook, WebhookEvent
+from whisperer.events import registry
+from whisperer.models import EventQueue, Webhook, WebhookEvent
 from whisperer.validators import countdown_kwargs_serializers
 
 
@@ -31,6 +32,13 @@ class WebhookSerializer(serializers.ModelSerializer):
         model = Webhook
         exclude = ('user',)
 
+    def validate_event_type(self, value):
+        try:
+            _ = registry[value]  # Check value in the registry keys
+            return value
+        except KeyError:
+            raise ValidationError('"%s" is not a valid choice.' % value)
+
 
 class WebhookEventSerializer(serializers.ModelSerializer):
     webhook = WebhookSerializer()
@@ -38,3 +46,9 @@ class WebhookEventSerializer(serializers.ModelSerializer):
     class Meta:
         model = WebhookEvent
         fields = '__all__'
+
+
+class EventQueueSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = EventQueue
+        fields = '__all__'
