@@ -1,5 +1,5 @@
 from whisperer.exceptions import WebhookAlreadyRegistered
-from whisperer.models import Webhook
+from whisperer.models import EventQueue, Webhook
 
 
 class WebhookService(object):
@@ -37,3 +37,27 @@ class WebhookService(object):
     def delete_webhook(self, webhook):
         webhook.is_active = False
         webhook.save(update_fields=['is_active'])
+
+
+class EventQueueService(object):
+    @staticmethod
+    def create_event_queue(
+        hook_id,
+        event_type,
+        app_label=None,
+        model_name=None,
+        instance_pk=None,
+        instance_dict=None,
+    ):
+        if instance_dict is None:
+            instance_dict = {}
+        instance = EventQueue(
+            hook_id=hook_id,
+            event_type=event_type,
+            app_label=app_label,
+            model_name=model_name,
+            instance_pk=instance_pk,
+            instance_dict=instance_dict,
+        )
+        instance.save()
+        return instance
