@@ -1,5 +1,6 @@
 import json
 import logging
+from datetime import timedelta
 
 import requests
 from celery import current_app
@@ -12,12 +13,14 @@ from django.utils.module_loading import import_string
 from rest_framework.utils.encoders import JSONEncoder
 
 from whisperer.client import WebhookClient
+from whisperer.conf import settings
 from whisperer.events import registry
 from whisperer.exceptions import (
     EventAlreadyDelivered,
     UnknownEventType,
     WebhookEventDoesNotExist,
 )
+from whisperer.services import EventQueueService
 from whisperer.utils import LockTask
 
 TASK_RETRY_COUNT = 10
@@ -37,6 +40,7 @@ def deliver_event_task(
     self,
     hook_id,
     event_type,
+    event_queue_uuid=None,
     event_uuid=None,
     instance=None,
     app_label=None,
@@ -45,16 +49,50 @@ def deliver_event_task(
     retry=True,
     **kwargs
 ):
-    from whisperer.models import Webhook
+    if not event_uuid and not (pk or instance):
+        raise ValueError("Both pk and instance can not be null")
+
+    from whisperer.models import EventQueue, Webhook
+
+    if event_queue_uuid:
+        exists = EventQueue.objects.filter(uuid=event_queue_uuid).exists()
+        if not exists:
+            # This means task is duplicate and already executed
+            return
 
     hook = Webhook.objects.get(pk=hook_id)
     if not event_uuid and not instance and app_label and model_name and pk:
-        model_class = apps.get_model(app_label, model_name)
-        instance = model_class.objects.get(pk=pk)
+        object_deleted = False
+        event_class = registry[event_type]
+        if event_class.queryset:
+            try:
+                instance = event_class.queryset.get(pk=pk)
+            except event_class.queryset.model.DoesNotExist:
+                object_deleted = True
+        else:
+            model_class = apps.get_model(app_label, model_name)
+            try:
+                instance = model_class.objects.get(pk=pk)
+            except model_class.DoesNotExist:
+                object_deleted = True
+        if object_deleted:
+            # Object deleted before the task started
+            if event_queue_uuid:
+                # clear event_queue object so event can't be triggered
+                # indefinitely
+                EventQueue.objects.filter(uuid=event_queue_uuid).delete()
+
+            return
 
     webhook_event, response = _deliver_event(
-        hook, instance, event_type, event_uuid=event_uuid
+        hook,
+        instance,
+        event_type,
+        event_uuid=event_uuid,
+        event_queue_uuid=event_queue_uuid,
     )
+    # Set back the original event_queue_uuid None before retrying
+    self.request.kwargs['event_queue_uuid'] = None
 
     if not response.ok:
         self.request.kwargs['event_uuid'] = webhook_event.uuid
@@ -70,10 +108,12 @@ def deliver_event_task(
         self.retry(countdown=hook.countdown.get_value(self.request.retries))
 
 
-def _deliver_event(hook, instance, event_type, event_uuid=None, force=False):
+def _deliver_event(
+    hook, instance, event_type, event_uuid=None, event_queue_uuid=None, force=False
+):
     from django.contrib.contenttypes.models import ContentType
 
-    from whisperer.models import WebhookEvent
+    from whisperer.models import EventQueue, WebhookEvent
 
     if event_type not in registry:
         raise UnknownEventType()
@@ -95,6 +135,7 @@ def _deliver_event(hook, instance, event_type, event_uuid=None, force=False):
             'event': {'type': event_type, 'uuid': webhook_event.uuid.hex},
             'payload': serialize_instance,
         }
+        payload = event.customize_payload(payload)
 
     request_datetime = timezone.now()
     response = requests.Response()
@@ -135,6 +176,8 @@ def _deliver_event(hook, instance, event_type, event_uuid=None, force=False):
             else 1
         )
         webhook_event.save()
+        if event_queue_uuid:
+            EventQueue.objects.filter(uuid=event_queue_uuid).delete()
 
     if hook.callback:
         callback_function = import_string(hook.callback)
@@ -146,6 +189,7 @@ def _deliver_event(hook, instance, event_type, event_uuid=None, force=False):
 def deliver_event(instance, event_type, async_=True, event_uuid=None):
     from whisperer.models import Webhook
 
+    queue_service = EventQueueService()
     hooks = Webhook.objects.filter(event_type=event_type, is_active=True)
     for hook in hooks:
         if not async_:
@@ -153,18 +197,30 @@ def deliver_event(instance, event_type, async_=True, event_uuid=None):
             continue
         if isinstance(instance, (models.Model, models.base.ModelBase)):
             app_label, model_name = get_natural_key(instance)
+            event_queue_instance = queue_service.create_event_queue(
+                hook_id=hook.pk,
+                event_type=event_type,
+                app_label=app_label,
+                model_name=model_name,
+                instance_pk=instance.pk,
+            )
             deliver_event_task.delay(
                 hook_id=hook.pk,
                 event_type=event_type,
+                event_queue_uuid=event_queue_instance.uuid,
                 app_label=app_label,
                 model_name=model_name,
                 pk=instance.pk,
                 event_uuid=event_uuid,
             )
         elif isinstance(instance, dict):
+            event_queue_instance = queue_service.create_event_queue(
+                hook_id=hook.pk, event_type=event_type, instance_dict=instance
+            )
             deliver_event_task.delay(
                 hook_id=hook.pk,
                 event_type=event_type,
+                event_queue_uuid=event_queue_instance.uuid,
                 instance=instance,
                 event_uuid=event_uuid,
             )
@@ -190,3 +246,47 @@ def undelivered_event_scanner():
                 event_uuid=undelivered_event.uuid,
                 retry=False,
             )
+
+
+@current_app.task()
+def trigger_event_queue_events():
+    from whisperer.models import EventQueue
+
+    delayed_time_seconds = settings.WHISPERER_EVENT_QUEUE_DELAY_SECOND
+    delayed_time = timezone.now() - timezone.timedelta(seconds=delayed_time_seconds)
+    event_queues = EventQueue.objects.filter(modified_date__lt=delayed_time).all()
+    for event_queue in event_queues:
+        if event_queue.instance_pk:
+            deliver_event_task.delay(
+                hook_id=event_queue.hook_id,
+                event_type=event_queue.event_type,
+                event_queue_uuid=event_queue.uuid,
+                app_label=event_queue.app_label,
+                model_name=event_queue.model_name,
+                pk=event_queue.instance_pk,
+                event_uuid=None,
+            )
+        else:
+            deliver_event_task.delay(
+                hook_id=event_queue.hook_id,
+                event_type=event_queue.event_type,
+                event_queue_uuid=event_queue.uuid,
+                instance=event_queue.instance_dict,
+                event_uuid=None,
+            )
+
+
+@current_app.task(base=LockTask)
+def delete_outdated_webhook_events(older_than_weeks=12, limit=10000):
+    from whisperer.models import WebhookEvent
+
+    threshold_date = timezone.now() - timedelta(weeks=older_than_weeks)
+
+    outdated_webhook_items = WebhookEvent.objects.filter(
+        modified_date__lte=threshold_date
+    )[:limit]
+
+    try:
+        WebhookEvent.objects.filter(id__in=outdated_webhook_items).delete()
+    except Exception as exc:
+        logger.error(str(exc))
