@@ -1,5 +1,12 @@
+from distutils.version import StrictVersion
+
+import django
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+
+if StrictVersion(django.get_version()) >= StrictVersion("2.0.0"):
+    from django.utils.translation import gettext_lazy as _
+else:
+    from django.utils.translation import ugettext_lazy as _
 
 
 class Customer(models.Model):
