@@ -9,6 +9,7 @@ from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.test import TestCase, override_settings
 from django.utils import timezone
+from django_redis import get_redis_connection
 from model_mommy import mommy
 
 from whisperer.countdown import (
@@ -18,17 +19,20 @@ from whisperer.countdown import (
     RandomRetryCountdown,
 )
 from whisperer.exceptions import WebhookAlreadyRegistered
-from whisperer.models import Webhook, WebhookEvent
+from whisperer.models import EventQueue, Webhook, WebhookEvent
 from whisperer.resources.serializers import WebhookSerializer
 from whisperer.services import WebhookService
 from whisperer.tasks import (
     MAX_RETRY_COUNT,
     TASK_RETRY_COUNT,
+    delete_outdated_webhook_events,
     deliver_event,
     deliver_event_task,
+    trigger_event_queue_events,
     undelivered_event_scanner,
 )
 from whisperer.tests.models import Address, Customer, Order
+from whisperer.tests.serializers import OrderSerializer
 from whisperer.utils import LockTask
 
 
@@ -148,6 +152,8 @@ class WebhookTestCase(TestCase):
 
 @override_settings(CELERY_ALWAYS_EAGER=True)
 class WhispererEventTestCase(TestCase):
+    redis_connection = get_redis_connection("default")
+
     def setUp(self):
         self.target_url = "http://example.com/order_create"
         self.user = mommy.make(User, username="test_user")
@@ -188,6 +194,10 @@ class WhispererEventTestCase(TestCase):
             shipping_amount="1",
         )
 
+    def tearDown(self):
+        self.redis_connection.flushall()
+        super(WhispererEventTestCase, self).tearDown()
+
     def test_deliver_event(self):
         with requests_mock.Mocker() as mock:
             mock.register_uri(
@@ -235,6 +245,17 @@ class WhispererEventTestCase(TestCase):
             deliver_event(self.order, "order-created", event_uuid=webhookevent.uuid)
             webhookevent.refresh_from_db()
             self.assertTrue(webhookevent.delivered)
+            self.assertFalse(EventQueue.objects.exists())
+
+    @mock.patch("whisperer.tasks.deliver_event_task")
+    def test_deliver_event_async_creates_event_queue_object(self, m):
+        m.return_value = True
+        with requests_mock.Mocker() as mock:
+            mock.register_uri(
+                "POST", self.webhook.target_url, text="Order Created", status_code=200
+            )
+            deliver_event(self.order, "order-created")
+            self.assertTrue(EventQueue.objects.exists())
 
     @mock.patch("requests.post")
     def test_http_error(self, post_mock):
@@ -246,8 +267,9 @@ class WhispererEventTestCase(TestCase):
         self.assertEqual(webhook_events[0].response_http_status, 500)
         self.assertIn("HTTPError", webhook_events[0].response_content)
 
+    @mock.patch("whisperer.tasks.logger.exception")
     @mock.patch("requests.post")
-    def test_unboundlocal_error(self, post_mock):
+    def test_unboundlocal_error(self, post_mock, logger_mock):
         post_mock.side_effect = UnboundLocalError()
         self.order.save()
         webhook_events = WebhookEvent.objects.all()
@@ -255,6 +277,7 @@ class WhispererEventTestCase(TestCase):
         self.assertFalse(webhook_events[0].delivered)
         self.assertEqual(webhook_events[0].response_http_status, 500)
         self.assertIn("", webhook_events[0].response_content)
+        self.assertTrue(logger_mock.is_called)
 
     def test_undelivered_event_scanner(self):
         date1 = timezone.now() - timedelta(days=1)
@@ -414,6 +437,7 @@ class WhispererEventTestCase(TestCase):
     @requests_mock.mock()
     @mock.patch.object(LockTask, "is_exists_cache_key")
     def test_auth_config(self, m, mock_method):
+        m.register_uri("POST", self.target_url)
         mock_method.return_value = False
         self.order.save()
         self.order.number = "123"
@@ -434,11 +458,152 @@ class WhispererEventTestCase(TestCase):
         self.assertEqual(event.response_http_status, 401)
         self.assertFalse(event.delivered)
 
+    def test_triggering_already_deleted_event_queue(self):
+        self.order.save()
+        app_label, model_name = ContentType.objects.get_for_model(Order).natural_key()
+        instance_pk = self.order.pk
+        event_type = "order_created"
+        event_queue_uuid = uuid.uuid4()
+        # Reproducing Event Queue deleted with other celery task
+        # This means we might register same event more than once
+
+        old_webhook_event_count = WebhookEvent.objects.count()
+
+        with requests_mock.Mocker() as mock:
+            mock.register_uri(
+                "POST", self.webhook.target_url, text="Order Created", status_code=200
+            )
+            deliver_event_task(
+                hook_id=self.webhook.id,
+                event_type=event_type,
+                event_queue_uuid=event_queue_uuid,
+                app_label=app_label,
+                model_name=model_name,
+                pk=instance_pk,
+                event_uuid=None,
+            )
+
+        new_webhook_event_count = WebhookEvent.objects.count()
+        self.assertEqual(old_webhook_event_count, new_webhook_event_count)
+
+    def test_deliver_event_task_none_operation_control(self):
+        with self.assertRaises(ValueError):
+            deliver_event_task(
+                hook_id=self.webhook.id,
+                event_type="order_created",
+                intance=None,
+                pk=None,
+            )
+
+    def test_trigger_event_queue_events(self):
+        self.order.save()
+        # newly registered WebhookEvent
+        new_record = mommy.make(EventQueue)
+        # registered WebhookEvents idle more than 1 hour
+        app_label, model_name = ContentType.objects.get_for_model(Order).natural_key()
+        lost_record_1 = mommy.make(
+            EventQueue,
+            hook_id=self.webhook.pk,
+            event_type="order-created",
+            app_label=app_label,
+            model_name=model_name,
+            instance_pk=self.order.pk,
+        )
+        order_data = OrderSerializer(self.order).data
+        lost_record_2 = mommy.make(
+            EventQueue,
+            hook_id=self.webhook.pk,
+            event_type="order-created",
+            instance_dict=order_data,
+        )
+        expire_date = timezone.now() - timedelta(days=2)
+        EventQueue.objects.filter(
+            uuid__in=[lost_record_1.uuid, lost_record_2.uuid]
+        ).update(modified_date=expire_date)
+
+        WebhookEvent.objects.all().delete()
+
+        with requests_mock.Mocker() as mock:
+            mock.register_uri(
+                "POST",
+                self.target_url,
+                [{"text": "Order event received", "status_code": 200}],
+            )
+            trigger_event_queue_events()
+
+        event_queues_uuids = EventQueue.objects.values_list("uuid", flat=True)
+        self.assertEqual(len(event_queues_uuids), 1)
+        self.assertIn(new_record.uuid, event_queues_uuids)
+
+        self.assertEqual(WebhookEvent.objects.count(), 2)
+
+    def test_object_deleted_before_deliver_event_started(self):
+        app_label, model_name = ContentType.objects.get_for_model(Order).natural_key()
+        lost_record_1 = mommy.make(
+            EventQueue,
+            hook_id=self.webhook.pk,
+            event_type="order-created",
+            app_label=app_label,
+            model_name=model_name,
+            instance_pk=1231243,  # Produce does not exists object
+        )
+        lost_record_2 = mommy.make(
+            EventQueue,
+            hook_id=self.webhook.pk,
+            event_type="order-updated",
+            app_label=app_label,
+            model_name=model_name,
+            instance_pk=123121898,  # Produce does not exists object
+        )
+        expire_date = timezone.now() - timedelta(days=2)
+        EventQueue.objects.filter(
+            uuid__in=[lost_record_1.uuid, lost_record_2.uuid]
+        ).update(modified_date=expire_date)
+
+        with requests_mock.Mocker() as mock:
+            mock.register_uri(
+                "POST",
+                self.target_url,
+                [{"text": "Order event received", "status_code": 200}],
+            )
+            trigger_event_queue_events()
+
+        event_queues_uuids = EventQueue.objects.values_list("uuid", flat=True)
+        self.assertEqual(len(event_queues_uuids), 0)
+        # Because object is deleted there are no WebhookEvent
+        self.assertEqual(WebhookEvent.objects.count(), 0)
+
     def _auth_test_callback(self, request, context):
         if "Authorization" not in request.headers:
             context.status_code = 401
         return ""
 
+    def test_delete_outdated_webhook_events(self):
+        events = mommy.make(
+            WebhookEvent,
+            request_payload={},
+            webhook=mommy.make(Webhook, retry_countdown_config={}),
+            _quantity=30,
+        )
+        threshold_date = timezone.now() - timedelta(weeks=1)
+        outdated_events = [webhook.id for webhook in events[:10]]
+
+        # set outdated_webhook_events' modified_date to before threshold_date
+        WebhookEvent.objects.filter(id__in=outdated_events).update(
+            modified_date=threshold_date - timedelta(days=1)
+        )
+        self.assertEqual(WebhookEvent.objects.count(), 30)
+
+        delete_outdated_webhook_events(older_than_weeks=1)
+
+        self.assertFalse(
+            WebhookEvent.objects.filter(
+                id__in=outdated_events,
+            ).exists()
+        )
+        # Events that are not outdated should not be deleted
+        self.assertEqual(WebhookEvent.objects.count(), 20)
+
 
 def dummy_whisperer_event_callback(response, event_type, instance, payload):
     """this function creates customer in test database"""
