@@ -1,4 +1,4 @@
-from django.db.models.signals import post_save
+from django.db.models.signals import post_save, pre_delete
 
 from whisperer.events import WhispererEvent, registry
 from whisperer.tasks import deliver_event
@@ -9,6 +9,10 @@ from whisperer.tests.serializers import OrderSerializer
 class OrderCreateEvent(WhispererEvent):
     serializer_class = OrderSerializer
     event_type = 'order-created'
+    queryset = Order.objects.select_related(
+        "customer",
+        "address",
+    ).all()
 
 
 class OrderUpdateEvent(WhispererEvent):
@@ -16,8 +20,14 @@ class OrderUpdateEvent(WhispererEvent):
     event_type = 'order-updated'
 
 
+class OrderDeleteEvent(WhispererEvent):
+    serializer_class = OrderSerializer
+    event_type = 'order-deleted'
+
+
 registry.register(OrderCreateEvent)
 registry.register(OrderUpdateEvent)
+registry.register(OrderDeleteEvent)
 
 
 def signal_receiver(instance, created=False, **kwargs):
@@ -27,4 +37,11 @@ def signal_receiver(instance, created=False, **kwargs):
         deliver_event(instance, 'order-updated')
 
 
+def signal_delete_receiver(sender, instance, created=False, **kwargs):
+    order_data = OrderSerializer(instance).data
+
+    deliver_event(order_data, 'order-deleted')
+
+
 post_save.connect(signal_receiver, Order)
+pre_delete.connect(signal_delete_receiver, sender=Order)
