@@ -5,11 +5,16 @@ except ImportError:
 
 from rest_framework.routers import DefaultRouter
 
-from whisperer.resources.views import WebhookEventViewSet, WebhookViewSet
+from whisperer.resources.views import (
+    EventQueueViewSet,
+    WebhookEventViewSet,
+    WebhookViewSet,
+)
 
 router = DefaultRouter()
 router.register(r'hooks', WebhookViewSet)
 router.register(r'hook_events', WebhookEventViewSet)
+router.register(r'event_queues', EventQueueViewSet)
 
 app_name = 'whisperer'
 
